#
# Copyright (C) 2013-2013, Intercom Srl, Daniele Orlandi
#
# Author:: Daniele Orlandi <daniele@orlandi.com>
#          Lele Forzani <lele@windmill.it>
#          Alfredo Cerutti <acerutti@intercom.it>
#
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.
#

module Ygg
module Provisioner


#class API < Tomte::Network::API::Server
#  def console_connected
#    {
#     :message => "Connected to #{self.class.name.underscore} version #{::PhpHostingProvisioner::VERSION}",
#     :version => worker.class.version,
#     :class_name => self.class.name,
#    }
#  end
#
#  def version
#    worker.class.version
#  end
#
#  def help
#    # For all methods gather help
#  end
#
#end










class Worker < Tomte::Worker

  cattr_accessor :version
  cattr_accessor :subsystem

  cattr_accessor :agentcfg
  cattr_accessor :exname
  cattr_accessor :task_producer

  cattr_accessor :task_class

  def self.init_defaults
    self.agentcfg = config.send(subsystem)
    self.exname = [ agentcfg[:exchange_prefix], agentcfg[:subsystem], 'agent', agentcfg[:nodename] ].join('.')
  end

  bus :service

  private
  #######

  def init(options = {})
    task_class.worker = self
    task_class.config = config
    task_class.agentcfg = agentcfg
    task_class.log = log

    @endpoint = endpoint(:service)

    @endpoint.rpc_server(
      :api => exname + '.api',
      :raw => true,
      :queue_options => { :auto_delete => true },
#      :exchange_options => { :auto_delete => true } ## THIS BREAKS CONSOLE
      :server => proc { |verb, request|
        task_class.new(
          :operation => verb,
          :request_data => request.payload.parameters,
          :also_publish_to => nil,
        ).execute do |res|
          request.return(res)
        end

        throw :_async
      }
    ).ready!

    #task_class.endpoint = @endpoint

    self.task_producer = @endpoint.producer(
      :exchange => agentcfg[:task_messages_exchange],
      :exchange_options => {
        :passive => true,
        :auto_delete => false,
        :durable => true
      },
      :reliable => true
    )

    @consumer = @endpoint.consumer(
      :queue => exname,
      :queue_options => {
        :durable => true,
        :auto_delete => false,
      },
      :prefetch => 1,
    )

    @consumer.on_delivery do |metadata, message|

      message[:also_publish_to] = task_producer

      begin
        task_class.new(message) do |task|
          task.execute do
            metadata.ack
          end
        end
      rescue Ygg::Provisioner::Task::OperationNotDefined => e
        if message.also_publish_to
          message.also_publish_to.publish({
            :msg_type => :PERMANENT_FAILURE,
            :request_id => message.uuid,
            :log => e.to_s,
            },
            :persistent => true,
            :errback => lambda { |a| raise PublishError }) do
            metadata.ack
          end
        else
          metadata.ack
        end
      end
    end

    task_producer.connect! do
      @consumer.connect! do |c|
        c.declare_exchange exname, :direct, { :durable => true, :auto_delete => true } do |exchange|
          c.bind(exchange)
          c.consume do
            log.info "ready to receive messages"
          end
        end
      end
    end
  end
end

end
end
